# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-15 06:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0026_online_payments'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='coursejournalentry',
            new_name='journalentry',
        ),
        migrations.AlterField(
            model_name='journalentry',
            name='registrations',
            field=models.ManyToManyField(blank=True, related_name='journal_entries', to='leprikon.SubjectRegistration', verbose_name='participants'),
        ),
        migrations.RenameField(
            model_name='journalentry',
            old_name='course',
            new_name='subject',
        ),
        migrations.AlterField(
            model_name='journalentry',
            name='subject',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, related_name='journal_entries', to='leprikon.Subject', verbose_name='subject'),
        ),
        migrations.RenameModel(
            old_name='coursejournalleaderentry',
            new_name='journalleaderentry',
        ),
        migrations.AlterModelOptions(
            name='journalleaderentry',
            options={'verbose_name': 'journal leader entry', 'verbose_name_plural': 'journal leader entries'},
        ),
        migrations.RenameField(
            model_name='journalleaderentry',
            old_name='course_entry',
            new_name='journal_entry',
        ),
        migrations.AlterField(
            model_name='journalleaderentry',
            name='journal_entry',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='leader_entries', to='leprikon.JournalEntry', verbose_name='journal entry'),
        ),
        migrations.AlterField(
            model_name='journalleaderentry',
            name='timesheet',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, related_name='journal_entries', to='leprikon.Timesheet', verbose_name='timesheet'),
        ),
    ]
