# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2022-02-09 08:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0065_subject_type_page"),
    ]

    operations = [
        migrations.AlterField(
            model_name="subjecttype",
            name="slug",
            field=models.SlugField(unique=True),
        ),
    ]
