# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-26 00:05
from __future__ import unicode_literals

from django.db import migrations, models
from django.utils.timezone import now


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0022_bankreader'),
    ]

    operations = [
        migrations.CreateModel(
            name='AccountClosure',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('closure_date', models.DateField(db_index=True, verbose_name='closure date')),
            ],
            options={
                'verbose_name': 'account closure',
                'verbose_name_plural': 'account closures',
                'ordering': ('-closure_date',),
            },
        ),
        migrations.RenameField(
            model_name='coursediscount',
            old_name='created',
            new_name='accounted',
        ),
        migrations.RenameField(
            model_name='eventdiscount',
            old_name='created',
            new_name='accounted',
        ),
        migrations.RenameField(
            model_name='subjectpayment',
            old_name='created',
            new_name='accounted',
        ),
        migrations.AlterModelOptions(
            name='coursediscount',
            options={'ordering': ('accounted',), 'verbose_name': 'course discount', 'verbose_name_plural': 'course discounts'},
        ),
        migrations.AlterModelOptions(
            name='eventdiscount',
            options={'ordering': ('accounted',), 'verbose_name': 'event discount', 'verbose_name_plural': 'event discounts'},
        ),
        migrations.AlterModelOptions(
            name='subjectpayment',
            options={'ordering': ('accounted',), 'verbose_name': 'payment', 'verbose_name_plural': 'payments'},
        ),
        migrations.AlterField(
            model_name='coursediscount',
            name='accounted',
            field=models.DateTimeField(default=now, verbose_name='accounted time'),
        ),
        migrations.AlterField(
            model_name='eventdiscount',
            name='accounted',
            field=models.DateTimeField(default=now, verbose_name='accounted time'),
        ),
        migrations.AlterField(
            model_name='subjectpayment',
            name='accounted',
            field=models.DateTimeField(default=now, verbose_name='accounted time'),
        ),
    ]
